/**** A P P L I C A T I O N   N O T E   ************************************
*
* Title			: DMX512 transmission library
* Version		: v1.1
* Last updated	: 30.07.07
* Target		: Transceiver Rev.3.01 [ATmega8515]
* Clock			: 8MHz, 16MHz
*
* written by hendrik hoelscher, www.hoelscher-hi.de
***************************************************************************
 This program is free software; you can redistribute it and/or 
 modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version2 of 
 the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 General Public License for more details. 

 If you have no copy of the GNU General Public License, write to the 
 Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

 For other license models, please contact the author.

***************************************************************************/

#include "globals.h"
#include "lib_dmx_out.h"

// ********************* local definitions *********************
		uint16_t 	gCurDmxCh;		//current DMX channel

#define TX_BRK  (0xFFFF)
#define TX_SC	(0)
#define TX_DATA (1)


// *************** DMX Reception Initialisation ****************
void init_DMX_TX(void)
{
//Ports
DDRD  |= (1<<2)|(1<<1);
PORTD |= (1<<2)|(1<<1);				//enable transmission

//USART
UBRRH  = 0;
UBRRL  = ((F_OSC/4000)-1);			//250kbaud, 8N2
UCSRC  = (1<<URSEL)|(3<<UCSZ0)|(1<<USBS);
UCSRB  = (1<<TXEN)|(1<<TXCIE);
UDR    = 0;							//start USART

//Data
gCurDmxCh= TX_BRK;					//start with break
}



// ****************** DMX Transmission ISR ********************

ISR (UART_TX_vect)
{
uint16_t DmxCh= gCurDmxCh;

if (DmxCh == TX_BRK)						//start break
	{
	UBRRH  = 0;
	if (Flags &(1<<SLOW_TX))
		{
		UBRRL  = 17;
		}
	else
		{
		UBRRL  = 9;
		UCSRC  = (1<<URSEL)|(3<<UCSZ0);		//8n1
		}
	UDR    = 0;								//send break
	gCurDmxCh= TX_SC;
	}
else if (DmxCh == TX_SC)
	{
	UBRRH  = 0;
	UBRRL  = ((F_OSC/4000)-1);				//250kbaud
	UCSRC  = (1<<URSEL)|(3<<UCSZ0)|(1<<USBS);
	UDR    = 0;								//send start byte
	gCurDmxCh= TX_DATA;
	}
else
	{
	if (Flags &(1<<SLOW_TX))
		{
		_delay_us(20);						//inter byte gap in slow mode
		}

	if (DmxCh &(1<<0)) UDR=  TxVal;			//send data
	else			   UDR= (0xFF -TxVal);
		
	gCurDmxCh++;
	if (Flags &(1<<SLOW_TX))
		{
		if (DmxCh == 511) gCurDmxCh= TX_BRK;
		}
	else
		{
		if (DmxCh == 48)  gCurDmxCh= TX_BRK;
		}
	}
}

